#********************************************************************************
# Create file CHANGELOG from git (commit descriptions)                          #
#                                                                               #
#  Author: Kai Peter (kp@qware.org), (c)2015-YYYY                               #
# Version: 2.6.1                                                                #
# License: ISC License                                                          #
#                                                                               #
# Description: Get log entries from git repository and create a CHANGELOG file. #
#              Only entries which are not in the CHANGELOG will be added. Thus  #
#              an existing old CHANGELOG will be kept.                          #
#              qube-chglog have to be executed inside the git repository.       #
#********************************************************************************
# standard configuration section
ME=`basename $0`
VERSION=`cat $0 | grep -m 1 'Version' | cut -d\   -f3`       # get version number

declare -r CHANGELOG="CHANGELOG"               # filename of the 'changelog' file
declare -r SAVEDHASH="../.qube/lastchangelog"  # last added commit hash (saved)
[ $DBG ] || DBG=0                              # set debug level (default: off)
LIBDIR="QLIBDIR"

declare -r SYSTYPE=`uname -s | tr 'A-Z' 'a-z'`
DATESTR=`date '+%Y%m%d'`
#--------------------------------------------------------------------------------

warn() { printf "%s" "$1"; return; }
error() { printf "\033[1m$ME: Error: %s Aborting!\033[0m\n" "$1"; exit 1; }

fmtGitDate() {
  # convert date from 'git log' into timestamp 'YYYYMMDD' for the CHANGELOG file
  local GITDATE=`echo $LINE | cut -d\  -f2-6`    # get date string from 'git log'

  case "$SYSTYPE" in
    freebsd)
       GITDATE=`date -j -f "%a %b %d %H:%M:%S %Y" "$GITDATE" '+%Y%m%d'`
       ;;
    netbsd|openbsd)
       GITDATE=`date -j -d "$GITDATE" '+%Y%m%d'`
       ;;
    *) GITDATE=`date -d "$GITDATE" +%Y%m%d`;;     # GNU 'date' only!
  esac

  # the first date we have to save in any case
  [ ! "$ACTDATE" ] && ACTDATE="$GITDATE" && echo "$ACTDATE:" >> "$CHANGELOG.new"
  # if the date in 'git log' changes
  [ "$ACTDATE" != "$GITDATE" ] && ACTDATE="$GITDATE" && \
      echo "$ACTDATE:" >> "$CHANGELOG.new"
  ISDATE=""
}

tmpfile() {       # write the number of lines given in $LNR to a temporarily file
  echo "Writing new commits to temporary file ..."

  local LH=""
  [ "$CLOG" ] && LH="$CLOG..HEAD"

  [ -n $TMP ] || [ -n $TEMP ] && TMP="/tmp"    # --> TODO: check logic
  local TMPFILE=`mktemp $TMP/gitlog-XXXXXXXXXXXX`
  git --no-pager log $LH | grep -v "^commit" | grep -v '^$' > $TMPFILE

  # read the tmp file
  while IFS="" read LINE ; do
    # check for a new date field
    LINE=`echo "$LINE" | sed 's/^[ \t]*//g'` # remove any leading spaces and tabs
    [ -z "$LINE" ] || ISDATE=`echo $LINE | cut -d\  -f1`
    [ "$ISDATE" = "Date:" ] && fmtGitDate
    # ignore some lines
    [ "$ISDATE" = "Author:" ] && ISDATE=""
    # a long $ISDATE has a value write all commit comments
    [ -n "$ISDATE" ]  && echo '  - '"$LINE" >> "$CHANGELOG.new" && ISDATE=""
  done <"$TMPFILE"
  [ "$DBG" -lt 1 ] && rm -f "$TMPFILE"     # the temp. file is no longer required
}

updateChangelog() {
  cd $SRCDIR || error "Unable to change into §SRCDIR"
  # some checks against the git repository
  [ ! -d ".git" ] && warn "not a git repository" && exit 1
  git status &>DEVNULL
  EC=$?; [ $EC -ne 0 ] && warn "not a git repository" && exit $EC
  git --no-pager log -n 1 &>DEVNULL
  EC=$?; [ $EC -ne 0 ] && warn "no commits in 'git log' found!" && exit $EC

  # get the last saved commit hash from file $SAVEDHASH
  printf "Last commit hash from $CHANGELOG: "
  [ -f "$SAVEDHASH" ] && CLOG=`head -1 "$SAVEDHASH"` && echo "$CLOG" || \
    echo "Nothing found! (assuming first time run)"

  # get last commit hash from 'git log'
  CGIT=`git --no-pager log -n 1 | head -1 | cut -d\  -f2`
  # print the hash of the last git commit
  echo "Last commit hash from 'git log':" $CGIT

  # compare the hashes
  [ "$CLOG" ] && [ "$CLOG" = "$CGIT" ] && \
    echo "$CHANGELOG is up-to-date!" && exit 0

  tmpfile     # call function tmpfile()

  # now concatenate an existing CHANGELOG file (ok, this code is a bit dirty)
  [ -f "$CHANGELOG" ] && if [ ! "$CLOG" ] ; then
  # insert some visual separation lines before an older changelog - begin -------
  cat >> "$CHANGELOG.new" << EOF

********************************************************************
 Below is the former/older changelog w/o git"

EOF
  # end separation lines ----------------------------------------------------------
  fi && cat "$CHANGELOG" >> "$CHANGELOG.new"

  # cleanup
  echo -n "Create new $CHANGELOG file ..."
  [ "$DBG" -lt 2 ] && mv "$CHANGELOG.new" "$CHANGELOG"
  if [ "$CGIT" ] ; then               # (re)write the lastchangelog file
    echo "$CGIT"  > $SAVEDHASH        # save the latest commit hash ...
    echo ""      >> $SAVEDHASH        # ... and add some comment
    echo "WARNING! Do not delete this file!"               >> $SAVEDHASH
    echo "The first line here is the latest commit hash"   >> $SAVEDHASH
    echo "from 'git log' which was attended to $CHANGELOG" >> $SAVEDHASH
  fi
  echo " done!"
  cd $OLDPWD
}
getTopQubeDir                 # top level directory where we start any processing

CFGFILE=".qube/config"           # the config file HAVE TO exist in this location
[ ! -f "$CFGFILE" ] && error "No config file found!" || source "$CFGFILE"

# some default settings (fallback, usually set in config file)
[ ! $SRCDIR ] && SRCDIR="src"
[ ! -d "$SRCDIR" ] && error "Unable to determine sources directory!"

#echo "Checking changelog status"
updateChangelog
